;(function(document, $, History) {
    "use strict";
    // in-page navigation with browser history API is not supported for Submissions page
    History.enabled = false;

    var DEFAULT_PER_PAGE = 25;
    var SORT_NAME_ATTRIBUTE = 'smartling-sort-name';
    var DIRECTION_MAP = {
        'ascending': 'ASC',
        'descending': 'DESC',
        'default': 'ASC'
    };

    var pagesPerCount = DEFAULT_PER_PAGE;

    var $tableBody;
    var $filterButton;
    var $displayingCount;
    var $currentPage;
    var $prevPage;
    var $nextPage;
    var $mySubmissionsCheckbox;
    var $submitterField;

    //filters
    var $title;
    var $path;
    var $status;
    var localesField;

    /*
    {
      "progress": 0,
      "wordCount": 0,
      "targetPath": "/content/cq:tags/properties",
      "state": "In translation",
      "locale": "zh",
      "nodePath": "/etc/smartling/submissions/content/cq:tags/properties/zh",
      "sourcePath": "/content/cq:tags/properties",
      "submitterId": "admin",
      "jcrTitle": "/content/cq:tags/properties",
      "fileUri": "/content/cq:tags/properties-11264_zh.xml",
      "configType": "TAG",
      "applied": null,
      "submitted": "23.10.2018 00:30",
      "target_locale": "Chinese (*)"
     }
     */
    var rowTemplate = _.template(
        '<tr is="coral-table-row" data-locale="<%= locale %>" data-file-uri="<%= fileUri %>" data-node-path="<%= nodePath %>" data-config-type="<%= configType %>" data-state="<%= state %>" data-target-path="<%= targetPath %>" data-source-path="<%= sourcePath %>">' +
            '<td is="coral-table-cell"> <coral-checkbox coral-table-rowselect></coral-checkbox> </td>' +
            '<td is="coral-table-cell"> <%= titleItem %> </td>' +
            '<td is="coral-table-cell"> <%= fileUri %> </td>' +
            '<td is="coral-table-cell"> <%= target_locale %> </td>' +
            '<td is="coral-table-cell"> <%= sourcePath %> </td>' +
            '<td is="coral-table-cell"> <%= submitterId %> </td>' +
            '<td is="coral-table-cell"> <%= state %> </td>' +
            '<td is="coral-table-cell"> <%= submitted %> </td>' +
            '<td is="coral-table-cell"> <%= applied %> </td>' +
            '<td is="coral-table-cell"> <%= progress %> </td>' +
            '<td is="coral-table-cell"> <%= wordCount %> </td>' +
        '</tr>'
    );

    var titleHrefTpl = _.template('<a target="_blank" href="<%= previewLink %>"> <%= jcrTitle %> </a>');

    const tplDisplayingCount = _.template('Displaying <%= start %>-<%= end %> of <%= total %>');

    const selectLocaleTemplate = _.template('<coral-select-item value="<%= locale %>"><%= country %></coral-select-item>');

    $(document).on("foundation-contentloaded", function(e){
        $tableBody = $('tbody[is=coral-table-body]');

        $title = $('#title-filter');
        $path = $('#path-filter');
        $displayingCount = $('#displaying-count');
        $status = $('coral-select#states');
        localesField =  CUI.Widget.fromElement('Autocomplete', $('#locale-filter'));

        initPagination();
        initPageCountSelect();
        initFilterButton();

        initSorting();
        initSelectingRows();

        initSubmitterFilter();
        loadData({});
    });

    function initPagination() {
        $currentPage = $('#current-page');
        $currentPage.keyup(function(event) {
            if (event.keyCode === 13) {
                loadData({});
            }
        });

        $prevPage = $('#previous-page');
        $prevPage.on('click', function() {
            var updatePageNum = $currentPage.val() - 1;
            $currentPage.val(updatePageNum);
            loadData({});
        });

        $nextPage = $('#next-page');
        $nextPage.on('click', function() {
            var updatePageNum = parseInt($currentPage.val()) + 1;
            $currentPage.val(updatePageNum);
            loadData({});
        })
    }

    function initPageCountSelect() {

        $('#items-per-page').on('change', function(event) {
            pagesPerCount = $('#items-per-page').val();
            loadData({});
        });
    }

    function initFilterButton() {
        $filterButton = $('#filter-action-button');
        $filterButton.on('click', function() {
            $currentPage.val(1);
            loadData({});
        });
    }

    function initSorting() {
        $('#table_custom_sort').on('coral-table:beforecolumnsort', function(event) {
            var element = event.detail.column;
            var sortName = element.getAttribute(SORT_NAME_ATTRIBUTE);
            var direction = event.detail.direction;

            var sortParameters = {'sort': sortName, 'dir': DIRECTION_MAP[direction]};

            loadData(sortParameters);
        });
    }

    function initSelectingRows() {
        $('#table_custom_sort').on('coral-table:change', function(event) {
            var eventDetails = event.detail;

            if (eventDetails.oldSelection.length == 0 && event.detail.selection.length > 0) {
                $(document).trigger('foundation-mode-change', 'selection', '');
            }

            if (eventDetails.oldSelection.length > 0 && event.detail.selection.length == 0) {
                $(document).trigger('foundation-mode-change', 'default', '');
            }
        });
    }

    function initSubmitterFilter() {
        $submitterField = $('#submitter-filter');
        $mySubmissionsCheckbox = $('#my-submissions-filter');
        $mySubmissionsCheckbox.on('change', function(event) {
            if (event.target.checked) {
                $submitterField.prop('disabled', true);
            } else {
                $submitterField.prop('disabled', false);
            }
        });
    }

    function loadData(sortParameters) {
        $tableBody.empty();
        $.ajax({
            url: '/services/smartling/submission/fileStatus.json?' + buildParameters(sortParameters)
        }).done(function(data) {
            data.records.forEach(function(item) {
                item.progress = Math.floor(item.progress * 100) + '%';
                var parameters = $.extend({}, item, {titleItem: buildTitleItem(item)});
                $tableBody.append(rowTemplate(parameters));
            });

            updateDisplayingCount(data.results);
            updatePagination(data.results);
        })
    }

    function buildParameters(sortParameters) {

        var params = {'_dc' :(new Date().getTime())};

        params['start'] = pagesPerCount * ($currentPage.val() - 1);
        params['limit'] = pagesPerCount;
        params['titleField'] = $title.val();
        params['pathField'] = $path.val();

        var states = [];
        $status.find('coral-select-item[selected]').each(function(i, item) {
            states.push(item.value);
        });

        if (!!states && states.length > 0) {
            params['states'] = states;
        }

        var locales = localesField.getValue(); // getValue() returns array of selected locale codes (e.g. ['zh', 'de'])
        if (!!locales && locales.length > 0) {
            params['locale'] = locales;
        }

        var mySubmissions = $mySubmissionsCheckbox.attr('checked');

        if (!!mySubmissions) {
            params['mysubmissions'] = !!mySubmissions;
        } else {
            params['submitter'] = $submitterField.val();
        }

        return $.param($.extend({}, params, sortParameters));
    }

    function buildTitleItem(item) {

        switch(item.configType) {
            case 'CONTENT':
                return titleHrefTpl({previewLink: '/editor.html' + item.targetPath + '.html', jcrTitle: item.jcrTitle});

            case 'TAG':
                return titleHrefTpl({previewLink: '/libs/cq/tagging/gui/content/tags.html' + item.sourcePath, jcrTitle: item.jcrTitle});

            default:
                return item.jcrTitle;
        }

    }

    function updateDisplayingCount(resultsCount) {
        var start = ($currentPage.val() - 1) * pagesPerCount + 1;
        var end = $currentPage.val() * pagesPerCount;
        $displayingCount.empty().append(tplDisplayingCount({
            'start': start,
            'end':  end >= resultsCount && resultsCount > start ? resultsCount : end,
            'total': resultsCount < start ? end : resultsCount
        }));
    }
    function updatePagination(resultsCount) {
        var disableNext = false;
        var disablePrev = $currentPage.val() == 1;
        $nextPage.prop('disabled', disableNext);
        $prevPage.prop('disabled', disablePrev);
    }

})(document, Granite.$, window.History);