CQ.Ext.ns("CQ.Smartling");

CQ.Smartling.Dialog = CQ.Ext.extend(CQ.Ext.Window, {
    form: null,
    invalidMessage: "",

    constructor: function (config) {
        var focusIndex = 0;

        if (config.description) {
            config.items.unshift(CQ.Smartling.Dialog.createDescription(config.description));
            delete config.description;
            focusIndex = 1;
        }
        // apply focus index if defined by dialog
        focusIndex = config.focusIndex ? config.focusIndex : focusIndex;

        var formCfg = config.formCfg ? config.formCfg : {};
        formCfg.items = config.items;


        this.form = new CQ.Ext.form.FormPanel(CQ.Ext.applyIf(formCfg, CQ.Smartling.Dialog.FORM_DEFAULTS));
        delete config.items;

        CQ.Ext.applyIf(config, {
            y: 120,
            draggable: true,
            modal: true,
            layout: "fit",
            height: 565,
            width: 534,
            resizable: true,
            title: "Edit Configuration",
            closable: false,
            items: this.form,

            buttons: [
                {
                    text: "Test connection",
                    width: 100,
                    handler: this.testConnection,
                    scope: this,
                    formBind: true
                },
                {
                    text: "OK",
                    scale: "small",
                    handler: this.save,
                    scope: this,
                    formBind: true
                },
                {
                    text: "Cancel",
                    scale: "small",
                    handler: function () {
                        this.cancel();
                    },
                    scope: this
                }
            ],

            listeners: {
                show: function (win) {
                    win.form.items.get(focusIndex).focus(true, 400);
                    win.onShow();
                }
            }
        });
        CQ.Smartling.Dialog.superclass.constructor.call(this, config);
    },

    initComponent: function () {
        CQ.Smartling.Dialog.superclass.initComponent.call(this);

        this.addEvents(
            /**
             * @event success
             * Fires when the dialog was successfully submitted. What this means
             * depends on the dialog implementation. Typically this is a success
             * response from the dialog's form submission.
             * @param {CQ.Smartling.Dialog} this dialog
             * @param {Object} args... further arguments depending on the implementation
             */
            "success",

            /**
             * @event failure
             * Fires when the dialog submission failed. What this means
             * depends on the dialog implementation. Typically this is a failure
             * response from the dialog's form submission.
             * @param {CQ.Smartling.Dialog} this dialog
             * @param {Object} args... further arguments depending on the implementation
             */
            "failure"
        );

    },

    onShow: function () {
        // can be overwritten by subclasses
    },

    // private
    save: function () {
        var dlg = this;

        var form = this.form.getForm();
        if (form.isValid()) {
            if (this.doSubmit()) {
                dlg.hide();
            }
        }
    },

    doSubmit: function () {
        // override to implement your submit actions
    },

    getTargetInfos: function (panelItems) {
        // override in subclass
    },

    getContextParams: function (panelItems) {
        // no context by default
        return undefined;
    },

    getTargets: function () {
        return undefined;
    },

    getSourceLocalPath: function () {
        var panelItems = this.items.items[0].items.items[0];
        return panelItems.items.items[1].sourceLocalePath.getValue();
    },

    getSourcePath: function () {
        var panelItems = this.items.items[0].items.items[0];
        return panelItems.items.items[1].sourcePath.getValue();
    },

    _checkEquality: function (comparable, anotherComparable, message) {
        if (comparable === anotherComparable) {
            this.invalidMessage = this.invalidMessage + message;
            return false;
        }

        return true;
    },

    _validateTargetFields: function (valid) {

        var targetInfoList = this.getTargets();
        var sourceLocalePath = this.getSourceLocalPath();
        var sourcePath = this.getSourcePath();

        for (var i = 0; i < targetInfoList.length; i++) {
            var targetInfo = JSON.parse(targetInfoList[i]);

            valid = valid && this._checkEquality(targetInfo.cqLocalePath, sourceLocalePath,
                "Error for field targetInfo: 'One of the selected target locales is equal to source locale' <br/>");

            // for dictionary target path undefined
            if (targetInfo.targetPath) {
                valid = valid && this._checkEquality(targetInfo.targetPath, sourcePath,
                    "Error for field targetInfo: 'One of the selected target paths is equal to source path' <br/>");
            }

            for (var j = i + 1; j < targetInfoList.length; j++) {
                var anotherTargetInfo = JSON.parse(targetInfoList[j]);

                // for dictionary target path undefined
                if (targetInfo.targetPath) {
                    valid = valid && this._checkEquality(targetInfo.targetPath, anotherTargetInfo.targetPath,
                        "Error for field targetInfo: 'One of the target paths is equal to another path' <br/>");
                }

                valid = valid && this._checkEquality(targetInfo.smartlingLocale, anotherTargetInfo.smartlingLocale,
                    "Error for field targetInfo: 'One of the Smartling Locales is equal to another locale' <br/>");

                valid = valid && this._checkEquality(targetInfo.cqLocalePath, anotherTargetInfo.cqLocalePath,
                    "Error for field targetInfo: 'One of the Adobe CQ Locales is equal to another locale' <br/>");
            }
        }

        return valid;
    },

    getLocale: function (targetInfoValue) {
        var targetInfo = JSON.parse(targetInfoValue);
        var smartlingLocale = targetInfo.smartlingLocale;
        var cqLocale = targetInfo.cqLocalePath;
        cqLocale = cqLocale.substring(cqLocale.lastIndexOf("/") + 1);
        cqLocale = cqLocale.replace("_", "-");
        return smartlingLocale.length > 0 ? smartlingLocale : cqLocale;
    },

    testConnection: function () {
        var panelItems = this.items.items[0].items.items[0];

        var targetInfoValues = this.getTargetInfos(panelItems);
        var userSecretParam = panelItems.items.items[0].generalUserSecret.getValue();
        var userIdParam = panelItems.items.items[0].generalUserId.getValue();
        var projectIdParam = panelItems.items.items[0].generalProjectId.getValue();
        var localeParams = [];
        for (var i = 0; i < targetInfoValues.length; i++) {
            var locale = this.getLocale(targetInfoValues[i]);
            localeParams.push(locale);
        }

        var encodedParams = CQ.Ext.encode(
            {
                api: {
                    userSecret: userSecretParam,
                    userId: userIdParam,
                    projectId: projectIdParam,
                    locales: localeParams
                },
                context: this.getContextParams(panelItems)
            }
        );

        var loadMask = new CQ.Ext.LoadMask(this.el, {
            'msg': CQ.I18n.getMessage('Checking configuration...')
        });

        loadMask.show();

        CQ.Ext.Ajax.request({
            url: "/services/translation/testconnection",
            params: {
                testConnectionParams: encodedParams
            },
            callback: function () {
                loadMask.hide();
            },
            success: function (response) {
                var responseJson = CQ.Ext.decode(response.responseText);
                var apiMessagesEmpty = responseJson.apiMessages.length === 0;
                var contextMessagesEmpty = responseJson.contextTested ? responseJson.contextMessages.length === 0 : true;
                var successfulTest = (apiMessagesEmpty && contextMessagesEmpty);
                if (!successfulTest) {
                    var apiMessages = "Smartling API test:<br/>" + (apiMessagesEmpty ? "Successful" : responseJson.apiMessages.join(" <br/>")) + "<br/><br/>";
                    var contextMessages = "Context test:<br/> " + (contextMessagesEmpty ? "Successful" : responseJson.contextMessages.join(" <br/>")) + "<br/>";
                    CQ.Ext.Msg.show({
                        title: "Connection Error: ",
                        width: 500,
                        msg: apiMessages + (responseJson.contextTested ? contextMessages : ""),
                        buttons: CQ.Ext.MessageBox.OK,
                        icon: CQ.Ext.MessageBox.ERROR
                    });
                } else {
                    CQ.Ext.Msg.show({
                        title: "Success",
                        width: 300,
                        msg: "Connections tests were successful",
                        buttons: CQ.Ext.MessageBox.OK,
                        icon: CQ.Ext.MessageBox.INFO
                    });
                }
            },
            failure: function (response) {
                CQ.Ext.Msg.show({
                    title: "Connection Error: " + response.status,
                    width: 300,
                    msg: response.statusText,
                    buttons: CQ.Ext.MessageBox.OK,
                    icon: CQ.Ext.MessageBox.ERROR
                });
            }
        });

    },


    ajaxFormSubmit: function () {
        var dlg = this;
        this.form.getForm().submit({
            //url: this.form.url,
            timeout: 60,
            waitMsg: "submitting...",
            success: function (form, action) {
                dlg.fireEvent("success", dlg);
            },
            failure: CQ.Smartling.Dialog.DEFAULT_ERROR_HANDLER,
            scope: this
        });
        return true;
    },
    cancel: function () {
        // can be overwritten by subclasses
        this.hide();
    },
    reset: function () {
        this.form.getForm().reset();
    }
});
CQ.Smartling.Dialog.FORM_DEFAULTS = {
    border: false,
    /*bodyStyle:"padding:15px",*/
    autoScroll: true,
    defaults: {
    }
};

CQ.Smartling.Dialog.createDescription = function (description) {
    return {
        xtype: "box",
        autoEl: {
            tag: "div",
            cls: "dialog-text",
            html: description
        },
        style: {
            marginBottom: "12px"
        }
    };
};

CQ.Smartling.Dialog.DEFAULT_ERROR_HANDLER = function (form, action) {
    switch (action.failureType) {
        case CQ.Ext.form.Action.CONNECT_FAILURE:
            CQ.Ext.Msg.show({
                title: "Could not connect to server",
                msg: action.result.msg,
                icon: CQ.Ext.MessageBox.ERROR,
                buttons: CQ.Ext.MessageBox.OK
            });
            break;
        default:
            CQ.Ext.Msg.show({
                title: "Error",
                msg: action.result.msg,
                icon: CQ.Ext.MessageBox.ERROR,
                buttons: CQ.Ext.MessageBox.OK
            });
    }
    this.fireEvent("failure", this, action.result);
};