CQ.Smartling.DigitalAssetTargetInfoPanel = CQ.Ext.extend(CQ.form.CompositeField, {
    cqLocalePath: null,
    smartlingLocale: null,
    targetPath: null,

    constructor: function (config) {
        config = config || {};

        var defaults = {
            "labelWidth": 50,
            "labelPad": 0,
            "layout": "fit",
            "padding": 10
        };

        config = CQ.Util.applyDefaults(config, defaults);
        CQ.Smartling.DigitalAssetTargetInfoPanel.superclass.constructor.call(this, config);
    },

    initComponent: function () {
        CQ.Smartling.DigitalAssetTargetInfoPanel.superclass.initComponent.call(this);

        digitalAssetTargetInfoPanel = new CQ.form.CompositeField({
            layout: "form",
            border: false,
            autoHeight: true
        });
        this.targetPath = new CQ.form.PathField({
            fieldLabel: "Target Path",
            name: "targetPath",
            width: 190,
            rootPath: "content",
            value: this.targetPath,
            selectOnFocus: true,
            editable: false
        });
        digitalAssetTargetInfoPanel.add(this.targetPath);

        this.cqLocalePath = new CQ.form.Selection({
            fieldLabel: "Adobe CQ Locale",
            name: "cqLocalePath",
            width: 190,
            type: "select",
            options: "/services/translation/cqlocales",
            optionsRoot: "cqLocales",
            optionsTextField: "label",
            optionsValueField: "path",
            allowBlank: false,
            msgTarget: "qtip"
        });
        this.cqLocalePath.setValue("");
        digitalAssetTargetInfoPanel.add(this.cqLocalePath);

        this.smartlingLocale = new CQ.Ext.form.TextField({
            fieldLabel: "Smartling Locale",
            name: "smartlingLocale",
            allowBlank: false,
            blankText: "Smartling locale is required.",
            width: 190
        });
        digitalAssetTargetInfoPanel.add(this.smartlingLocale);
        this.add(digitalAssetTargetInfoPanel);
    },

    processInit: function (path, record) {
        this.cqLocalePath.processInit(path, record);
        this.smartlingLocale.processInit(path, record);
        this.targetPath.processInit(path, record);
    },

    setValue: function (value) {
        var targetInfo = JSON.parse(value);
        this.cqLocalePath.setValue(targetInfo.cqLocalePath);
        this.smartlingLocale.setValue(targetInfo.smartlingLocale);
        this.targetPath.setValue(targetInfo.targetPath);
        this.validate();
    },

    getValue: function () {
        return this.getRawValue();
    },

    getRawValue: function () {
        var targetInfo = {
            "cqLocalePath": this.cqLocalePath.getValue(),
            "smartlingLocale": this.smartlingLocale.getValue(),
            "targetPath": this.targetPath.getValue()
        };
        return JSON.stringify(targetInfo);
    }
});
CQ.Ext.reg("digitalassettargetinfo", CQ.Smartling.DigitalAssetTargetInfoPanel);

CQ.Smartling.DigitalAssetMappingDialog = CQ.Ext.extend(CQ.Smartling.Dialog, {
    form: null,

    constructor: function (config) {
        CQ.Ext.applyIf(config, {
            formCfg: {
                labelWidth: 80,
                labelPad: 0,
                defaults: {
                    padding: 15,
                    width: 520,
                    height: 515
                }
            },
            items: [
                {
                    xtype: "tabpanel",
                    activeTab: 0,
                    items: [
                        {
                            title: "General settings",
                            items: [
                                {
                                    xtype: "fieldset",
                                    height: 455,
                                    defaults: {
                                        xtype: "textfield",
                                        width: 375
                                    },
                                    items: [
                                        {
                                            ref: "../configId",
                                            name: "configId",
                                            xtype: "hidden"
                                        },
                                        {
                                            fieldLabel: "Type",
                                            ref: "../configType",
                                            name: "configType",
                                            readOnly: true
                                        },
                                        {
                                            fieldLabel: "Name",
                                            ref: "../configName",
                                            name: "configName",
                                            allowBlank: false
                                        },
                                        {
                                            fieldLabel: "Project Id",
                                            ref: "../generalProjectId",
                                            name: "generalProjectId",
                                            allowBlank: false
                                        },
                                        {
                                            fieldLabel: "User Identifier",
                                            ref: "../generalUserId",
                                            name: "generalUserId",
                                            allowBlank: false
                                        },
                                        {
                                            fieldLabel: "User Secret",
                                            ref: "../generalUserSecret",
                                            name: "generalUserSecret",
                                            allowBlank: false
                                        },
                                        {
                                            fieldLabel: "Callback URL",
                                            ref: "../callbackUrl",
                                            name: "callbackUrl",
                                            vtype: 'url'
                                        },
                                        {
                                            fieldLabel: "Retrieval Type",
                                            ref: "../retrievalType",
                                            name: "retrievalType",
                                            value: "published",
                                            allowBlank: "false",
                                            type: "select",
                                            xtype: "selection",
                                            options: "/services/translation/retrievaltypes",
                                            optionsRoot: "retrievalTypes",
                                            optionsTextField: "label",
                                            optionsValueField: "value"
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            title: "Adobe CQ settings",
                            items: [
                                {
                                    xtype: "fieldset",
                                    autoScroll: true,
                                    defaults: {
                                        xtype: "textfield",
                                        width: 375
                                    },
                                    items: [
                                        {
                                            fieldLabel: "Source Path",
                                            id: "digitalAssetSourcePath",
                                            ref: "../sourcePath",
                                            name: "sourcePath",
                                            value: this.sourcePath,
                                            rootPath: "content",
                                            selectOnFocus: "true",
                                            editable: false,
                                            allowBlank: false,
                                            xtype: "pathfield"
                                        },
                                        {
                                            fieldLabel: "Source Locale",
                                            id: "digitalAssetSourceLocalePath",
                                            ref: "../sourceLocalePath",
                                            name: "sourceLocalePath",
                                            options: "/services/translation/cqlocales",
                                            optionsRoot: "cqLocales",
                                            optionsTextField: "label",
                                            optionsValueField: "path",
                                            allowBlank: false,
                                            msgTarget: "qtip",
                                            type: "select",
                                            xtype: "selection"
                                        },
                                        {
                                            fieldLabel: "Target Info",
                                            name: "digitalAssetTargetInfoPanel",
                                            ref: "../digitalAssetTargetInfoPanel",
                                            autoScroll: true,
                                            height: 370,
                                            padding: "5px 5px 0 5px",
                                            xtype: "panel",
                                            items: [
                                                {
                                                    name: "digitalAssetTargetInfo",
                                                    ref: "../digitalAssetTargetInfo",
                                                    orderable: false,
                                                    width: 350,
                                                    fieldConfig: {
                                                        xtype: "digitalassettargetinfo"
                                                    },
                                                    xtype: "multifield"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            title: "Digital assets settings",
                            items: [
                                {
                                    xtype: "fieldset",
                                    defaults: {
                                        xtype: "textfield",
                                        width: 375
                                    },
                                    items: [
                                        {
                                            fieldLabel: "File types",
                                            name: "fileTypesPanel",
                                            ref: "../fileTypesPanel",
                                            autoScroll: true,
                                            height: 375,
                                            padding: "5px 5px 0 5px",
                                            xtype: "panel",
                                            items: [
                                                {
                                                    type: "checkbox",
                                                    xtype: "selection",
                                                    name: "fileTypes",
                                                    ref: "../fileTypes",
                                                    options: "/services/translation/filetypes",
                                                    optionsRoot: "fileTypes",
                                                    optionsTextField: "label",
                                                    optionsValueField: "value",
                                                    allowBlank: false,
                                                    msgTarget: "qtip",
                                                    defaults: {
                                                        checked: "true"
                                                    }
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        });

        CQ.Smartling.DigitalAssetMappingDialog.superclass.constructor.call(this, config);
    },

    getTargets: function () {
        var panelItems = this.items.items[0].items.items[0];
        return panelItems.items.items[1].digitalAssetTargetInfoPanel.items.items[0].getValue();
    },

    isValid: function (form) {
        var dialog = this;
        var valid = true;
        form.items.each(function (f) {
            if (f.isCheckbox && !f.el) {
                return;
            }

            if (f.getName() == "digitalAssetTargetInfo" && f.getValue().length < 1) {
                dialog.invalidMessage = dialog.invalidMessage + "Error for field " + f.getName() + ": 'You must add at least one item' <br/>";
                valid = false;
            }

            if (!f.validate()) {
                if (f.getErrors) {
                    var errors = f.getErrors(f.getValue());
                    var startIndex = errors.length > 1 ? 1 : 0;
                    for (i = startIndex; i < errors.length; i++) {
                        dialog.invalidMessage = dialog.invalidMessage + "Error for field " + f.getName() + ": '" + errors[i] + "' <br/>";
                    }
                } else {
                    dialog.invalidMessage = dialog.invalidMessage + "Error for field " + f.getName() + ": '" + f.blankText + "' <br/>";
                }
                valid = false;
            }
        });

        valid = this._validateCustomerKeys(valid);
        valid = this._validateTargetFields(valid);

        return valid;
    },

    _validateCustomerKeys: function _checkProxySettings(valid) {
        var generalSettings = this.items.items[0].items.items[0].items.items[0];
        var generalProjectId = generalSettings.generalProjectId.getValue();
        var RegExp = /^\w{9}$/;
        var errorMessage = "Project Id wrong configured. It should look like 'XXXXXXXXX'. <br/>";
        valid = this._mutchRegexp(valid, generalProjectId, RegExp, errorMessage);

        var generalUserSecret = generalSettings.generalUserSecret.getValue();
        RegExp = /.{54,55}/;
        errorMessage = "User Secret key wrong configured. It should look like 'X{54,55}'. <br/>";
        valid = this._mutchRegexp(valid, generalUserSecret, RegExp, errorMessage);

        var generalUserId = generalSettings.generalUserId.getValue();
        RegExp = /.{30}/;
        errorMessage = "User Identifier key wrong configured. It should look like 'X{30}'. <br/>";
        valid = this._mutchRegexp(valid, generalUserId, RegExp, errorMessage);
        return valid;
    },

    _mutchRegexp: function _mutchRegexp(isValid ,variable, regexp, errorMessage) {
        if (!regexp.test(variable)) {
            isValid = false;
            this.invalidMessage += errorMessage;
        }
        return isValid;
    },

    save: function () {
        this.invalidMessage = "Next fields are filled incorrectly: <br/>";
        var form = this.form.getForm();
        if (!this.isValid(form)) {
            CQ.Ext.Msg.show({
                title: "Validation Error",
                msg: this.invalidMessage,
                buttons: CQ.Ext.MessageBox.OK,
                icon: CQ.Ext.MessageBox.ERROR
            });
            return;
        }

        var isNewConfig = !this.strorageRecord;
        var strorageRecord = isNewConfig ? new this.configStore.recordType({ key: '' }) : this.strorageRecord;
        var panelItems = this.items.items[0].items.items[0];

        strorageRecord.beginEdit();
        strorageRecord.set("configType", panelItems.items.items[0].configType.getValue());
        strorageRecord.set("configName", panelItems.items.items[0].configName.getValue());
        strorageRecord.set("generalProjectId", panelItems.items.items[0].generalProjectId.getValue());
        strorageRecord.set("generalUserSecret", panelItems.items.items[0].generalUserSecret.getValue());
        strorageRecord.set("generalUserId", panelItems.items.items[0].generalUserId.getValue());
        strorageRecord.set("callbackUrl", panelItems.items.items[0].callbackUrl.getValue());
        strorageRecord.set("retrievalType", panelItems.items.items[0].retrievalType.getValue());
        strorageRecord.set("sourcePath", panelItems.items.items[1].sourcePath.getValue());
        strorageRecord.set("sourceLocalePath", panelItems.items.items[1].sourceLocalePath.getValue());

        var fileTypesArray = [];
        var fileTypes = panelItems.items.items[2].fileTypesPanel.items.items[0].optionItems.items;

        var n = 0;
        for (i = 0; i < fileTypes.length; i++) {
            if (fileTypes[i].checked) {
                fileTypesArray[n] = fileTypes[i].inputValue;
                n++;
            }
        }
        strorageRecord.set("fileTypes", JSON.stringify(fileTypesArray));

        var targetInfoValues = panelItems.items.items[1].digitalAssetTargetInfoPanel.items.items[0].getValue();
        var targetInfos = [];
        for (i = 0; i < targetInfoValues.length; i++) {
            targetInfos[i] = JSON.parse(targetInfoValues[i]);
        }
        strorageRecord.set("targetInfos", JSON.stringify(targetInfos));

        strorageRecord.endEdit();

        if (isNewConfig) {
            this.configStore.insert(0, strorageRecord);
        }

        this.configStore.saveAll();
        this.hide();
    },

    addTranslations: function () {
        this.strorageRecord = null;

        var tabs = this.form.items.items[0].items;

        for (i = 0; i < tabs.length; i++) {
            this.initTab(i, this);
            tabs.items[i].items.items[0].items.each(function (f) {
                if (f.name != "fileTypesPanel" && f.name != "digitalAssetTargetInfoPanel") {
                    f.setValue("");
                }
            }, this);
        }

        var panelItems = this.items.items[0].items.items[0];
        panelItems.items.items[0].configType.setValue("DIGITAL_ASSET");
        panelItems.items.items[0].retrievalType.setValue("published");

        var fileTypes = panelItems.items.items[2].fileTypesPanel.items.items[0].optionItems.items;
        for (i = 0; i < fileTypes.length; i++) {
            if (fileTypes[i].el) {
                fileTypes[i].setValue(true);
            }
        }

        panelItems.items.items[1].digitalAssetTargetInfoPanel.items.items[0].setValue("");

        this.form.items.items[0].setActiveTab(0);
        this.show();
    },

    initTab: function (i, dialog) {
        dialog.form.items.items[0].setActiveTab(i);
        if (!dialog.rendered) {
            dialog.render(CQ.Util.getRoot());
        }
        dialog.beforeShow();
    },

    editTranslations: function (strorageRecord) {
        this.strorageRecord = strorageRecord;
        this._fillDialogFromRecord(strorageRecord);

    },

    cloneTranslations: function (strorageRecord) {
        this.strorageRecord = null;
        this._fillDialogFromRecord(strorageRecord);
    },

    _fillDialogFromRecord: function _fillDialogFromRecord(strorageRecord) {
        var tabs = this.form.items.items[0].items;

        var targetInfos = JSON.parse(strorageRecord.get("targetInfos"));
        for (i = 0; i < targetInfos.length; i++) {
            targetInfos[i] = JSON.stringify(targetInfos[i]);
        }
        for (i = 0; i < tabs.length; i++) {
            this.initTab(i, this);
            tabs.items[i].items.items[0].items.each(function (f) {
                if (this.strorageRecord == null && typeof f.name != "undefined" && f.name == "configName") {
                    f.setValue(strorageRecord.get(f.name) + " Copy");
                } if (typeof f.name != "undefined" && f.name != "fileTypesPanel" && f.name != "digitalAssetTargetInfoPanel") {
                    f.setValue(strorageRecord.get(f.name));
                } if (typeof f.name != "undefined" && f.name != "fileTypesPanel" && f.name == "digitalAssetTargetInfoPanel") {
                    f.items.items[0].setValue(targetInfos);
                }
            }, this);
        }

        var panelItems = this.items.items[0].items.items[0];
        var fileTypes = panelItems.items.items[2].fileTypesPanel.items.items[0].optionItems.items;
        var fileTypesJson = strorageRecord.get("fileTypes");

        for (i = 0; i < fileTypes.length; i++) {
            if (fileTypes[i].el) {
                fileTypes[i].setValue(fileTypesJson.indexOf(fileTypes[i].boxLabel) > 0);
            }
        }

        this.form.items.items[0].setActiveTab(0);

        this.show();
    },

    getTargetInfos: function (panelItems) {
        return panelItems.items.items[1].digitalAssetTargetInfoPanel.items.items[0].getValue();
    }
});
CQ.Ext.reg("smartlingDigitalAssetMappingdialog", CQ.Smartling.DigitalAssetMappingDialog);