CQ.Ext.ns('SMARTLING.ui.submission');

SMARTLING.ui.submission.SubmissionStore = CQ.Ext.extend(CQ.Ext.data.JsonStore, {
    defaultPageSize: 25,

    constructor: function () {
        SMARTLING.ui.submission.SubmissionStore.superclass.constructor.call(this, {
            proxy: new CQ.Ext.data.HttpProxy({
                url: '/services/smartling/submission/fileStatus.json',
                method: 'GET'
            }),
            baseParams: {
                start: 0,
                limit: this.defaultPageSize
            },
            root: 'records',
            fields: [
                'jcrTitle',
                'fileUri',
                'submissionType',
                'locale',
                'target_locale',
                'sourcePath',
                'targetPath',
                'updated',
                'submitted',
                'applied',
                'submitterId',
                'progress',
                'wordCount',
                'state',
                'submissionName',
                'configType',
                'nodePath'
            ],
            totalProperty: 'results',
            remoteSort: true,
            autoLoad: true
        });
    },

    setPageSize: function (pageSize) {
        this.baseParams.limit = pageSize;
    },

    setTitle: function (title) {
        this.baseParams.titleField = title;
    },

    setPath: function (pathField) {
        this.baseParams.pathField = pathField;
    },

    setStates: function (states) {
        if (states.length > 0) {
            this.baseParams.states = states;
        } else {
            delete  this.baseParams.states;
        }
    }
});

SMARTLING.ui.submission.SubmissionPagingToolbar = CQ.Ext.extend(CQ.Ext.PagingToolbar, {
    constructor: function (store) {
        var toolbar = this;
        SMARTLING.ui.submission.SubmissionPagingToolbar.superclass.constructor.call(this, {
            store: store,
            pageSize: store.defaultPageSize,
            displayInfo: true,
            items: [
                '-',
                'Per Page: ',
                new CQ.Ext.form.ComboBox({
                    width: 40,
                    store: new CQ.Ext.data.ArrayStore({
                        fields: ['id'],
                        data: [
                            ['10'],
                            ['15'],
                            ['25'],
                            ['50']
                        ]
                    }),
                    mode: 'local',
                    value: store.defaultPageSize,
                    listWidth: 40,
                    triggerAction: 'all',
                    displayField: 'id',
                    valueField: 'id',
                    editable: false,
                    forceSelection: true,
                    listeners: {
                        'select': function (combo, record) {
                            toolbar.pageSize = parseInt(record.get('id'), 10);
                            store.setPageSize(toolbar.pageSize);

                            toolbar.doLoad(toolbar.cursor);
                        }
                    }
                })
            ]
        });
    }
});


SMARTLING.ui.submission.SubmissionGrid = CQ.Ext.extend(CQ.Ext.grid.GridPanel, {
    constructor: function (config) {
        var grid = this;
        var theStore = new SMARTLING.ui.submission.SubmissionStore();
        var topToolBarConfigItems = [];
        for (var actionName in config.topToolbar) {
            var action = config.topToolbar[actionName];
            if (action && action.action) {
                topToolBarConfigItems.push(new SMARTLING.ui.common.TranslationAction(action, grid));
            }
        }

        function updateTopToolbarButtonState(model) {
            var selections = model.getSelections();
            for (i = 0; i < topToolBarConfigItems.length; i++) {
                if (selections.length <= 0) {
                    topToolBarConfigItems[i].disable();
                } else {
                    topToolBarConfigItems[i].enable();
                }
            }
        }


        var contextMenu = new CQ.Ext.menu.Menu({
            items: topToolBarConfigItems
        });

        SMARTLING.ui.submission.SubmissionGrid.superclass.constructor.call(this, {
            title: config.title,
            layout: 'fit',
            margins: '0 0 0 0',
            autoExpandColumn: 'fileUri',
            selModel: new CQ.Ext.grid.RowSelectionModel({
                listeners: {
                    'selectionchange': function (model) {
                        updateTopToolbarButtonState(model);
                    }
                }
            }),
            listeners: {
                'rowdblclick': function (grid, rowIndex) {
                    var data = grid.store.getAt(rowIndex).data;
                    if (data.targetPath === null || data.configType !== 'CONTENT') {
                        CQ.Ext.Msg.show({
                            title: 'Warning',
                            msg: 'Preview is not available',
                            icon: CQ.Ext.Msg.INFO,
                            buttons: CQ.Ext.Msg.OK
                        });
                    }
                    else {
                        window.open('/apps/smartling/preview.html?source=' + data.sourcePath + '&target=' + data.targetPath);
                    }

                },
                'rowcontextmenu': function (grid, rowIndex, e) {
                    e.stopEvent();
                    contextMenu.showAt(e.getXY());
                    return false;
                }
            },
            loadMask: true,
            cm: new CQ.Ext.grid.ColumnModel({columns: config.columns}),
            store: theStore,
            tbar: topToolBarConfigItems,
            bbar: new SMARTLING.ui.submission.SubmissionPagingToolbar(theStore)

        });
    },

    addBeforeReloadListener: function (listener) {
        this.getStore().addListener('beforeload', listener);
    },

    reload: function (title, path, states) {
        this.getStore().setTitle(title);
        this.getStore().setPath(path);
        this.getStore().setStates(states);
        this.getStore().load({});
    },

    reloadGrid: function () {
        this.getStore().load({});
    },

    _mask: function () {
        if (!this.loadMask) {
            this.loadMask = new CQ.Ext.LoadMask('CQ', {
                'msg': CQ.I18n.getMessage('Processing...')
            });
        }
        this.loadMask.show();
    },

    _unmask: function () {
        if (this.loadMask) {
            this.loadMask.hide();
        }
    }
});
